/**
* Name: Schelling
* Author: bgaudou
* Description: 
* Tags: Tag1, Tag2, TagN
*/

model Schelling3

global {	
	int nb_people <- 2000;
	float rate_similar_wanted <- 0.4;
	float neighbours_distance <- 5.0;	
	
	// Global variable computing the number of happy people
	int nb_happy_people update: people count each.is_happy;
	
	init {
		create people number: nb_people;
	}
	
	reflex end_simulation when: nb_happy_people = nb_people {
		write "End of the simulation";
		do pause;
	}
}

species people {	
	rgb color <- flip(0.5) ? #yellow : #red;
	bool is_happy;
	
	list<people> neighbours update: people at_distance neighbours_distance;
	
	reflex compute_similarity {
		float rate_similar <- 0.0;
		if(empty(neighbours)) {
			rate_similar <- 1.0;
		} else {
			int nb_neighbours <- length(neighbours);
			int nb_neighbours_sim <- neighbours count (each.color = color);
			rate_similar <- nb_neighbours_sim / nb_neighbours;
		}
		
		is_happy <- rate_similar >= rate_similar_wanted;
	}
	
	reflex move when: not(is_happy) {
		location <- any_location_in(world);
	}
	
	aspect circle {
		draw circle(2) color: color border: #black;
	}
}

experiment exp type: gui {
	parameter "Nb people agents" var: nb_people min: 1 max: 5000 step: 1;
	parameter "rate similar wanted" var: rate_similar_wanted min: 0.0 max: 1.0;
	parameter "neighbours distance" var: neighbours_distance step: 1.0;
	
	output {
		display d {
			species people aspect: circle;
		}
		
		display series_happy {
			chart "happy people" type: series {
				data "happy" value: nb_happy_people color: #green;
			}
		}
		
		monitor "Happy people" value: nb_happy_people;
	}
}